#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;

namespace BestFit
{
    public class BestFitEventArgs : EventArgs
    {
        protected int currentMonth;

        public BestFitEventArgs(int CurrentMonth)
        {
            currentMonth = CurrentMonth;
        }

        public int CurrentMonth
        {
            get { return currentMonth; }
            set { currentMonth = value; }
        }
    }

    public class BestFit
    {
        protected List<DebtorAccount> debtorAccountArray;
        protected List<List<DebtorAccount>> debtorAccountMonthArray;
        protected int currentMonth;
        protected bool bestFitCancel;

        protected int debtorProfileId;
        protected Enumerators.AdministrationType administrationType;

        protected decimal availableFunds;
        protected decimal monthlyFees;

        protected Dictionary<string, string> generalSettings;

        public event EventHandler<BestFitEventArgs> MonthCycleComplete;
        public event EventHandler<BestFitEventArgs> AccountCycleComplete;

        protected bool autoWeighting;
        protected bool adjustedWeighting;
        protected bool mustReweight;
        protected Enumerators.BestFitStyle bestFitStyle;

        protected int maximumRunningMonths;

        public BestFit()
        {
            debtorAccountArray = new List<DebtorAccount>();
            debtorAccountMonthArray = new List<List<DebtorAccount>>();

            generalSettings = new Dictionary<string, string>();

            debtorProfileId = -1;

            availableFunds = 0.00M;
            monthlyFees = 0.00M;

            currentMonth = 0;
            bestFitCancel = false;

            autoWeighting = true;
            adjustedWeighting = false;
            mustReweight = false;
            bestFitStyle = Enumerators.BestFitStyle.NoWeight;

            maximumRunningMonths = 100;
        }

        public void AddDebtorAccount(DebtorAccount NewDebtorAccount)
        {
            debtorAccountArray.Add(NewDebtorAccount);
        }

        public void AddDebtorAccountMonth(List<DebtorAccount> DebtorAccountArray)
        {
            debtorAccountMonthArray.Add(DebtorAccountArray);
        }

        public bool AutoWeight
        {
            get
            {
                return autoWeighting;
            }

            set
            {
                autoWeighting = value;
            }
        }

        public bool AdjustedWeight
        {
            get
            {
                return adjustedWeighting;
            }

            set
            {
                adjustedWeighting = value;
            }
        }

        public int MaximumRunningMonths
        {
            get { return maximumRunningMonths; }
            set { maximumRunningMonths = value; }
        }

        public bool MustWeight
        {
            get
            {
                return mustReweight;
            }

            set
            {
                mustReweight = value;
            }
        }

        public Enumerators.BestFitStyle BestFitStyle
        {
            get
            {
                return bestFitStyle;
            }

            set
            {
                bestFitStyle = value;
            }
        }

        public void Process()
        {
            decimal bestfitAvailableFunds = availableFunds;

            bestfitAvailableFunds -= monthlyFees;

            decimal recklessAvailableFunds = availableFunds;

            foreach (DebtorAccount da in this.DebtorAccountArray)
            {
                if (da.OriginalContractDate >= new DateTime(2007, 06, 01))
                {
                    recklessAvailableFunds -= da.OriginalInstallmentValue;

                    if (recklessAvailableFunds < 0.00M)
                    {
                        da.Reckless = true;
                    }
                }
            }

            decimal bondTotal = 0.00M;
            decimal carTotal = 0.00M;
            decimal retailTotal = 0.00M;
            decimal garnishTotal = 0.00M;
            decimal otherTotal = 0.00M;
            decimal categoryTotal = 0.00M;
            decimal bondCapitalTotal = 0.00M;
            decimal carCapitalTotal = 0.00M;
            decimal retailCapitalTotal = 0.00M;
            decimal garnishCapitalTotal = 0.00M;
            decimal otherCapitalTotal = 0.00M;
            decimal categoryCapitalTotal = 0.00M;
            foreach (DebtorAccount da in this.DebtorAccountArray)
            {
                switch (this.bestFitStyle)
                {
                    case Enumerators.BestFitStyle.Capital:
                        switch (da.AccountType)
                        {
                            case Enumerators.AccountType.HomeLoan:
                                bondCapitalTotal += da.CurrentSettlementValue;
                                bondTotal += da.OriginalInstallmentValue;
                                break;
                            case Enumerators.AccountType.CarLoan:
                                carCapitalTotal += da.CurrentSettlementValue;
                                carTotal += da.OriginalInstallmentValue;
                                break;
                            case Enumerators.AccountType.CreditorAccount:
                                retailCapitalTotal += da.CurrentSettlementValue;
                                retailTotal += da.OriginalInstallmentValue;
                                break;
                            case Enumerators.AccountType.GarnishOrder:
                                garnishCapitalTotal += da.CurrentSettlementValue;
                                garnishTotal += da.OriginalInstallmentValue;
                                break;
                            default:
                                otherCapitalTotal += da.CurrentSettlementValue;
                                otherTotal += da.OriginalInstallmentValue;
                                break;
                        }
                        break;
                    case Enumerators.BestFitStyle.Installment:
                    case Enumerators.BestFitStyle.Interest:
                        switch (da.AccountType)
                        {
                            case Enumerators.AccountType.HomeLoan:
                                bondTotal += da.OriginalInstallmentValue;
                                break;
                            case Enumerators.AccountType.CarLoan:
                                carTotal += da.OriginalInstallmentValue;
                                break;
                            case Enumerators.AccountType.CreditorAccount:
                                retailTotal += da.OriginalInstallmentValue;
                                break;
                            case Enumerators.AccountType.GarnishOrder:
                                garnishTotal += da.OriginalInstallmentValue;
                                break;
                            default:
                                otherTotal += da.OriginalInstallmentValue;
                                break;
                        }
                        break;
                    //case Enumerators.BestFitStyle.Interest:
                    //    switch (da.AccountType)
                    //    {
                    //        case Enumerators.AccountType.HomeLoan:
                    //            bondTotal += da.CurrentInterestRate;
                    //            break;
                    //        case Enumerators.AccountType.CarLoan:
                    //            carTotal += da.CurrentInterestRate;
                    //            break;
                    //        case Enumerators.AccountType.CreditorAccount:
                    //            retailTotal += da.CurrentInterestRate;
                    //            break;
                    //        case Enumerators.AccountType.GarnishOrder:
                    //            garnishTotal += da.CurrentInterestRate;
                    //            break;
                    //        default:
                    //            otherTotal += da.CurrentInterestRate;
                    //            break;
                    //    }
                    //    break;
                    case Enumerators.BestFitStyle.NoWeight:
                    default:
                        break;
                }
            }

            categoryTotal = bondTotal + carTotal + retailTotal + garnishTotal + otherTotal;
            categoryCapitalTotal = bondCapitalTotal + carCapitalTotal + retailCapitalTotal + garnishCapitalTotal + otherCapitalTotal;

            decimal bondAverageWeight = 0.00M;
            decimal carAverageWeight = 0.00M;
            decimal retailAverageWeight = 0.00M;
            decimal garnishAverageWeight = 0.00M;
            decimal otherAverageWeight = 0.00M;

            bondAverageWeight = bondTotal / categoryTotal;
            carAverageWeight = carTotal / categoryTotal;
            retailAverageWeight = retailTotal / categoryTotal;
            garnishAverageWeight = garnishTotal / categoryTotal;
            otherAverageWeight = otherTotal / categoryTotal;


            this.DebtorAccountMonthArray.Add(this.DebtorAccountArray);

            while ((hasOutstandingValue()) && (!bestFitCancel))
            {
                List<DebtorAccount> currentMonthDebtorAccountArray = new List<DebtorAccount>();

                currentMonthDebtorAccountArray = debtorAccountMonthArray[currentMonth];

                List<DebtorAccount> nextMonthDebtorAccountArray = new List<DebtorAccount>();

                decimal totalDebtValue = this.calculateTotalSettlement();
                decimal totalInstallmentValue = this.calculateTotalInstallment();
                decimal totalOriginalInstallmentValue = this.calculateTotalOriginalInstallment();
                decimal totalInterestValue = this.calculateTotalInterest();

                decimal installmentDifference = bestfitAvailableFunds - totalInstallmentValue;

                if (installmentDifference < 0.00M)
                {
                    installmentDifference = 0.00M;

                    totalInstallmentValue = bestfitAvailableFunds;
                }

                foreach (DebtorAccount currentMonthDebtorAccount in currentMonthDebtorAccountArray)
                {
                    if (autoWeighting)
                    {
                        if (totalDebtValue > 0)
                        {
                            //if (currentMonthDebtorAccount.CurrentSettlementValue == 0.00M)
                            //{
                            //    mustReweight = true;
                            //}

                            if (currentMonth == 0)
                            {
                                mustReweight = true;
                            }
                        }
                    }

                    if (mustReweight)
                    {
                        decimal weight = 0.00M;

                        switch (bestFitStyle)
                        {
                            case Enumerators.BestFitStyle.Capital:
                                switch (currentMonthDebtorAccount.AccountType)
                                {
                                    case Enumerators.AccountType.HomeLoan:
                                        weight = currentMonthDebtorAccount.CurrentSettlementValue / bondCapitalTotal;
                                        break;
                                    case Enumerators.AccountType.CarLoan:
                                        weight = currentMonthDebtorAccount.CurrentSettlementValue / carCapitalTotal;
                                        break;
                                    case Enumerators.AccountType.CreditorAccount:
                                        weight = currentMonthDebtorAccount.CurrentSettlementValue / retailCapitalTotal;
                                        break;
                                    case Enumerators.AccountType.GarnishOrder:
                                        weight = currentMonthDebtorAccount.CurrentSettlementValue / garnishCapitalTotal;
                                        break;
                                    default:
                                        weight = currentMonthDebtorAccount.CurrentSettlementValue / otherCapitalTotal;
                                        break;
                                }
                                break;
                            case Enumerators.BestFitStyle.Installment:
                                switch (currentMonthDebtorAccount.AccountType)
                                {
                                    case Enumerators.AccountType.HomeLoan:
                                        weight = currentMonthDebtorAccount.OriginalInstallmentValue / bondTotal;
                                        break;
                                    case Enumerators.AccountType.CarLoan:
                                        weight = currentMonthDebtorAccount.OriginalInstallmentValue / carTotal;
                                        break;
                                    case Enumerators.AccountType.CreditorAccount:
                                        weight = currentMonthDebtorAccount.OriginalInstallmentValue / retailTotal;
                                        break;
                                    case Enumerators.AccountType.GarnishOrder:
                                        weight = currentMonthDebtorAccount.OriginalInstallmentValue / garnishTotal;
                                        break;
                                    default:
                                        weight = currentMonthDebtorAccount.OriginalInstallmentValue / otherTotal;
                                        break;
                                }
                                break;
                            case Enumerators.BestFitStyle.Interest:
                                switch (currentMonthDebtorAccount.AccountType)
                                {
                                    case Enumerators.AccountType.HomeLoan:
                                        weight = totalInterestValue / bondTotal;
                                        break;
                                    case Enumerators.AccountType.CarLoan:
                                        weight = totalInterestValue / carTotal;
                                        break;
                                    case Enumerators.AccountType.CreditorAccount:
                                        weight = totalInterestValue / retailTotal;
                                        break;
                                    case Enumerators.AccountType.GarnishOrder:
                                        weight = totalInterestValue / garnishTotal;
                                        break;
                                    default:
                                        weight = totalInterestValue / otherTotal;
                                        break;
                                }
                                break;
                            case Enumerators.BestFitStyle.NoWeight:
                            default:
                                weight = 0.00M;
                                break;
                        }

                        currentMonthDebtorAccount.Weight = weight;

                        mustReweight = false;
                    }

                    decimal installmentValue = 0.00M;

                    if (currentMonthDebtorAccount.Weight != 0)
                    {
                        switch (currentMonthDebtorAccount.AccountType)
                        {
                            case Enumerators.AccountType.HomeLoan:
                                installmentValue = ((bestfitAvailableFunds * bondAverageWeight) * currentMonthDebtorAccount.Weight);
                                break;
                            case Enumerators.AccountType.CarLoan:
                                installmentValue = ((bestfitAvailableFunds * carAverageWeight) * currentMonthDebtorAccount.Weight);
                                break;
                            case Enumerators.AccountType.CreditorAccount:
                                installmentValue = ((bestfitAvailableFunds * retailAverageWeight) * currentMonthDebtorAccount.Weight);
                                break;
                            case Enumerators.AccountType.GarnishOrder:
                                installmentValue = ((bestfitAvailableFunds * garnishAverageWeight) * currentMonthDebtorAccount.Weight);
                                break;
                            default:
                                installmentValue = ((bestfitAvailableFunds * otherAverageWeight) * currentMonthDebtorAccount.Weight);
                                break;
                        }
                    }

                    DebtorAccount nextMonthDebtorAccount = new DebtorAccount();

                    decimal settlementValue = currentMonthDebtorAccount.CurrentSettlementValue;
                    installmentValue = installmentValue + (installmentDifference * currentMonthDebtorAccount.Weight);
                    decimal interestRate = currentMonthDebtorAccount.CurrentInterestRate;

                    bool inDuplum = Validators.ValidatorFactory.InDuplum(this.generalSettings, currentMonthDebtorAccount.OriginalContractValue, currentMonthDebtorAccount.TotalInterestValue);

                    if (settlementValue != 0)
                    {
                        decimal interestValue = (settlementValue * interestRate) / 12;

                        if (inDuplum)
                        {
                            interestValue = 0;
                        }

                        if (settlementValue < installmentValue)
                        {
                            installmentValue = settlementValue + interestValue;
                            //currentMonthDebtorAccount.CurrentInstallmentValue = installmentValue;
                            currentMonthDebtorAccount.InterestPayment = interestValue;
                        }

                        decimal closingBalance = (settlementValue + interestValue) - installmentValue;

                        nextMonthDebtorAccount.DebtorAccountId = currentMonthDebtorAccount.DebtorAccountId;
                        nextMonthDebtorAccount.DebtorProfileId = currentMonthDebtorAccount.DebtorProfileId;
                        nextMonthDebtorAccount.AccountStatus = currentMonthDebtorAccount.AccountStatus;
                        nextMonthDebtorAccount.AccountType = currentMonthDebtorAccount.AccountType;

                        nextMonthDebtorAccount.Weight = currentMonthDebtorAccount.Weight;

                        nextMonthDebtorAccount.OriginalContractValue = currentMonthDebtorAccount.OriginalContractValue;
                        nextMonthDebtorAccount.OriginalContractTerm = currentMonthDebtorAccount.OriginalContractTerm;
                        nextMonthDebtorAccount.OriginalContractDate = currentMonthDebtorAccount.OriginalContractDate;

                        nextMonthDebtorAccount.TotalFeeValue = currentMonthDebtorAccount.TotalFeeValue;
                        nextMonthDebtorAccount.TotalCapitalValue = currentMonthDebtorAccount.TotalCapitalValue;
                        nextMonthDebtorAccount.TotalCreditLifeValue = currentMonthDebtorAccount.TotalCreditLifeValue;
                        nextMonthDebtorAccount.TotalInterestValue = currentMonthDebtorAccount.TotalInterestValue;

                        nextMonthDebtorAccount.CurrentInterestRate = interestRate;
                        nextMonthDebtorAccount.CurrentInstallmentValue = installmentValue;
                        nextMonthDebtorAccount.InterestPayment = interestValue;
                        nextMonthDebtorAccount.TotalInterestValue = currentMonthDebtorAccount.TotalInterestValue + interestValue;
                        nextMonthDebtorAccount.CurrentSettlementValue = closingBalance;
                        nextMonthDebtorAccount.Month = currentMonthDebtorAccount.Month + 1;

                        try
                        {
                            nextMonthDebtorAccount.MonthDateTime = currentMonthDebtorAccount.MonthDateTime.AddMonths(1);
                        }
                        catch (System.Exception exception)
                        {
                            bestFitCancel = true;
                        }

                        nextMonthDebtorAccount.Reckless = currentMonthDebtorAccount.Reckless;
                        if (currentMonthDebtorAccount.InDuplum)
                        {
                            nextMonthDebtorAccount.InDuplum = currentMonthDebtorAccount.InDuplum;
                        }
                        else
                        {
                            nextMonthDebtorAccount.InDuplum = Validators.ValidatorFactory.InDuplum(this.generalSettings, nextMonthDebtorAccount.OriginalContractValue,
                                nextMonthDebtorAccount.TotalInterestValue);
                        }


                        nextMonthDebtorAccount.InDuplumValue = Validators.ValidatorFactory.InDuplumValue(this.generalSettings, nextMonthDebtorAccount.OriginalContractValue,
                            nextMonthDebtorAccount.TotalInterestValue);

                        if ((closingBalance == 0.00M) || (nextMonthDebtorAccount.CurrentSettlementValue == 0.00M))
                        {
                            nextMonthDebtorAccount.CurrentInstallmentValue = 0.00M;
                            nextMonthDebtorAccount.CurrentInterestRate = 0.00M;
                            nextMonthDebtorAccount.InterestPayment = 0.00M;
                        }

                        nextMonthDebtorAccountArray.Add(nextMonthDebtorAccount);

                        if (currentMonth == 0)
                        {
                            closingBalance = currentMonthDebtorAccount.CurrentSettlementValue;
                        }

                        currentMonthDebtorAccount.ClosingBalance = closingBalance;

                        if (AccountCycleComplete != null)
                        {
                            AccountCycleComplete(this, new BestFitEventArgs(currentMonth));
                        }
                    }
                }

                debtorAccountMonthArray.Add(nextMonthDebtorAccountArray);

                currentMonth++;

                if (currentMonth >= maximumRunningMonths)
                {
                    this.bestFitCancel = true;
                }

                if (MonthCycleComplete != null)
                {
                    MonthCycleComplete(this, new BestFitEventArgs(currentMonth));
                }
            }
        }

        public void ValidateDebtorAccount()
        {
            for (int i = 0; i < debtorAccountMonthArray.Count; i++)
            {
                List<DebtorAccount> currentMonthDebtorAccountArray = debtorAccountMonthArray[i];
                List<DebtorAccount> nextMonthDebtorAccount = new List<DebtorAccount>();

                DebtorAccount previousDebtorAccount = null;

                foreach (DebtorAccount currentMonthDebtorAccount in currentMonthDebtorAccountArray)
                {
                    if ((previousDebtorAccount != null) && (previousDebtorAccount.MaxContractInterestBreached))
                    {
                        currentMonthDebtorAccount.MaxContractInterestBreached = previousDebtorAccount.MaxContractInterestBreached;
                    }
                    else
                    {
                        currentMonthDebtorAccount.MaxContractInterestBreached = Validators.ValidatorFactory.MaxContractInterestBreached(this.generalSettings, currentMonthDebtorAccount.OriginalContractValue,
                        currentMonthDebtorAccount.TotalInterestValue, currentMonthDebtorAccount.CurrentInterestRate,
                        currentMonthDebtorAccount.OriginalContractDate);
                    }
                    if ((previousDebtorAccount != null) && (previousDebtorAccount.MaxInterestBreached))
                    {
                        currentMonthDebtorAccount.MaxInterestBreached = previousDebtorAccount.MaxInterestBreached;
                    }
                    else
                    {
                        currentMonthDebtorAccount.MaxInterestBreached = Validators.ValidatorFactory.MaxInterestBreached(this.generalSettings, currentMonthDebtorAccount.TotalInterestValue, currentMonthDebtorAccount.OriginalContractValue,
                        currentMonthDebtorAccount.OriginalContractTerm, currentMonthDebtorAccount.OriginalContractTerm, currentMonthDebtorAccount.AccountType);
                    }
                    if ((previousDebtorAccount != null) && (previousDebtorAccount.AverageInterestLargerThanContract))
                    {
                        currentMonthDebtorAccount.AverageInterestLargerThanContract = previousDebtorAccount.AverageInterestLargerThanContract;
                    }
                    else
                    {
                        currentMonthDebtorAccount.AverageInterestLargerThanContract = Validators.ValidatorFactory.AverageInterestLargerThanContract(this.generalSettings, currentMonthDebtorAccount.CurrentInterestRate,
                        currentMonthDebtorAccount.OriginalContractTerm, currentMonthDebtorAccount.OriginalContractValue);
                    }
                    if ((previousDebtorAccount != null) && (previousDebtorAccount.ContractTerm))
                    {
                        currentMonthDebtorAccount.ContractTerm = previousDebtorAccount.ContractTerm;
                    }
                    else
                    {
                        currentMonthDebtorAccount.ContractTerm = Validators.ValidatorFactory.ContractTerm(this.generalSettings, currentMonthDebtorAccount.OriginalContractDate,
                        currentMonthDebtorAccount.OriginalContractTerm);
                    }
                    if ((previousDebtorAccount != null) && (previousDebtorAccount.BestFitTerm))
                    {
                        currentMonthDebtorAccount.BestFitTerm = previousDebtorAccount.BestFitTerm;
                    }
                    else
                    {
                        currentMonthDebtorAccount.BestFitTerm = Validators.ValidatorFactory.BestFitTerm(this.generalSettings, currentMonth,
                        currentMonthDebtorAccount.OriginalContractDate,
                        currentMonthDebtorAccount.OriginalContractTerm);
                    }
                    if ((previousDebtorAccount != null) && (previousDebtorAccount.Balance))
                    {
                        currentMonthDebtorAccount.Balance = previousDebtorAccount.Balance;
                    }
                    else
                    {
                        currentMonthDebtorAccount.Balance = Validators.ValidatorFactory.Balance(this.generalSettings, currentMonthDebtorAccount.OriginalContractValue, currentMonthDebtorAccount.CreditLifePayment,
                        currentMonthDebtorAccount.OriginalContractTerm, currentMonthDebtorAccount.TotalInterestValue, currentMonthDebtorAccount.TotalPaidToDate,
                        currentMonthDebtorAccount.ClosingBalance);
                    }
                    if ((previousDebtorAccount != null) && (previousDebtorAccount.UnderAdministration))
                    {
                        currentMonthDebtorAccount.UnderAdministration = previousDebtorAccount.UnderAdministration;
                    }
                    else
                    {
                        currentMonthDebtorAccount.UnderAdministration = Validators.ValidatorFactory.UnderAdministration(this.generalSettings, this.administrationType);
                    }
                    if ((previousDebtorAccount != null) && (previousDebtorAccount.MaxFeesBreached))
                    {
                        currentMonthDebtorAccount.MaxFeesBreached = previousDebtorAccount.MaxFeesBreached;
                    }
                    else
                    {
                        currentMonthDebtorAccount.MaxFeesBreached = Validators.ValidatorFactory.MaxFeesBreached(this.generalSettings, currentMonthDebtorAccount.OriginalContractValue, currentMonthDebtorAccount.TotalFeeValue, currentMonthDebtorAccount.AccountType);
                    }

                    if ((previousDebtorAccount != null) && (previousDebtorAccount.BondMaxLengthExceeded))
                    {
                        currentMonthDebtorAccount.BondMaxLengthExceeded = previousDebtorAccount.BondMaxLengthExceeded;
                    }
                    else
                    {
                        currentMonthDebtorAccount.BondMaxLengthExceeded = Validators.ValidatorFactory.BondMaxLengthExceeded(GeneralSettings, currentMonthDebtorAccount.OriginalContractDate);
                    }

                    if ((previousDebtorAccount != null) && (previousDebtorAccount.CarMaxLengthExceeded))
                    {
                        currentMonthDebtorAccount.CarMaxLengthExceeded = previousDebtorAccount.CarMaxLengthExceeded;
                    }
                    else
                    {
                        currentMonthDebtorAccount.CarMaxLengthExceeded = Validators.ValidatorFactory.CarMaxLengthExceeded(GeneralSettings, currentMonthDebtorAccount.OriginalContractDate);
                    }

                    if ((previousDebtorAccount != null) && (previousDebtorAccount.RetailMaxLengthExceeded))
                    {
                        currentMonthDebtorAccount.RetailMaxLengthExceeded = previousDebtorAccount.RetailMaxLengthExceeded;
                    }
                    else
                    {
                        currentMonthDebtorAccount.RetailMaxLengthExceeded = Validators.ValidatorFactory.RetailMaxLengthExceeded(GeneralSettings, currentMonthDebtorAccount.OriginalContractDate);
                    }

                    if ((previousDebtorAccount != null) && (previousDebtorAccount.GarnishMaxLengthExceeded))
                    {
                        currentMonthDebtorAccount.GarnishMaxLengthExceeded = previousDebtorAccount.GarnishMaxLengthExceeded;
                    }
                    else
                    {
                        currentMonthDebtorAccount.GarnishMaxLengthExceeded = Validators.ValidatorFactory.GarnishMaxLengthExceeded(GeneralSettings, currentMonthDebtorAccount.OriginalContractDate);
                    }

                    currentMonthDebtorAccount.AverageInterestValueToSetAside = Validators.ValidatorFactory.AverageInterestLargerThanContractToSetAside(this.generalSettings, currentMonthDebtorAccount.CurrentInterestRate,
                            currentMonthDebtorAccount.OriginalContractTerm, currentMonthDebtorAccount.OriginalContractValue);
                    currentMonthDebtorAccount.ContractTermValue = Validators.ValidatorFactory.ContractTermLength(this.generalSettings, currentMonthDebtorAccount.OriginalContractDate,
                        currentMonthDebtorAccount.OriginalContractTerm);
                    currentMonthDebtorAccount.BestFitTermValue = Validators.ValidatorFactory.BestFitTermLength(this.generalSettings, currentMonth,
                        currentMonthDebtorAccount.OriginalContractDate,
                        currentMonthDebtorAccount.OriginalContractTerm);
                    currentMonthDebtorAccount.BalanceExpectedStatementBalance = Validators.ValidatorFactory.BalanceExpectedStatementValue(this.generalSettings, currentMonthDebtorAccount.OriginalContractValue, currentMonthDebtorAccount.CreditLifePayment,
                        currentMonthDebtorAccount.OriginalContractTerm, currentMonthDebtorAccount.TotalInterestValue, currentMonthDebtorAccount.TotalPaidToDate,
                        currentMonthDebtorAccount.ClosingBalance);
                    currentMonthDebtorAccount.MaxContractInterestActualInterestValue = Validators.ValidatorFactory.MaxContractInterestBreachedValue(this.generalSettings, currentMonthDebtorAccount.OriginalContractValue,
                        currentMonthDebtorAccount.TotalInterestValue, currentMonthDebtorAccount.CurrentInterestRate,
                        currentMonthDebtorAccount.OriginalContractDate);
                    currentMonthDebtorAccount.MaxFeesValue = Validators.ValidatorFactory.MaxFeesBreachedValue(this.generalSettings, currentMonthDebtorAccount.OriginalContractValue, currentMonthDebtorAccount.TotalFeeValue, currentMonthDebtorAccount.AccountType);
                    currentMonthDebtorAccount.MaxFeesExcessValue = Validators.ValidatorFactory.MaxFeesBreachedExcessValue(this.generalSettings, currentMonthDebtorAccount.OriginalContractValue, currentMonthDebtorAccount.TotalFeeValue, currentMonthDebtorAccount.AccountType);
                    currentMonthDebtorAccount.MaxFeesFeeValue = Validators.ValidatorFactory.MaxFeesBreachedFeeValue(this.generalSettings, currentMonthDebtorAccount.OriginalContractValue, currentMonthDebtorAccount.TotalFeeValue, currentMonthDebtorAccount.AccountType);
                    currentMonthDebtorAccount.MaxInterestValue = Validators.ValidatorFactory.MaxInterestBreachedValue(this.generalSettings, currentMonthDebtorAccount.TotalInterestValue, currentMonthDebtorAccount.OriginalContractValue,
                        currentMonthDebtorAccount.OriginalContractTerm, currentMonthDebtorAccount.OriginalContractTerm, currentMonthDebtorAccount.AccountType);

                    previousDebtorAccount = currentMonthDebtorAccount;
                }
            }
        }

        public void UpdateDebtorAccountValidation()
        {
            System.Data.SqlClient.SqlConnection connection = new SqlConnection(System.Configuration.ConfigurationManager.ConnectionStrings["EliyahuLiteConnectionString"].ConnectionString);

            connection.Open();

            System.Data.SqlClient.SqlCommand command = new SqlCommand("DELETE FROM Debtor.DebtorAccountValidation WHERE DebtorProfileId = @DebtorProfileId ",
                connection);

            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorProfileId", debtorProfileId));

            command.ExecuteNonQuery();

            connection.Close();

            List<DebtorAccount> debtorAccountArray = SumarizeDebtorAccount();

            if (debtorAccountMonthArray.Count >= 2)
            {
                debtorAccountArray = debtorAccountMonthArray[1];
            }
            else
            {
                debtorAccountArray = debtorAccountMonthArray[0];
            }

            foreach (DebtorAccount da in debtorAccountArray)
            {
                if (da.InDuplum)
                {
                    addDebtorAccountValidation("ID", da.DebtorAccountId, da.InDuplumValue.ToString(), null, null, null, null);
                }

                if ((da.MaxContractInterestBreached) || (da.MaxInterestBreached))
                {
                    addDebtorAccountValidation("MI", da.DebtorAccountId, da.MaxContractInterestActualInterestValue.ToString(), da.MaxInterestValue.ToString(), null, null, null);
                }

                if (da.AverageInterestLargerThanContract)
                {
                    addDebtorAccountValidation("AI", da.DebtorAccountId, da.AverageInterestValueToSetAside.ToString(), null, null, null, null);
                }

                if (da.ContractTerm)
                {
                    addDebtorAccountValidation("CT", da.DebtorAccountId, da.ContractTermValue.ToString(), null, null, null, null);
                }

                if (da.BestFitTerm)
                {
                    addDebtorAccountValidation("BT", da.DebtorAccountId, da.BestFitTermValue.ToString(), null, null, null, null);
                }

                if (da.Balance)
                {
                    addDebtorAccountValidation("BAL", da.DebtorAccountId, da.BalanceExpectedStatementBalance.ToString(), null, null, null, null);
                }

                if (da.UnderAdministration)
                {
                    addDebtorAccountValidation("UA", da.DebtorAccountId, null, null, null, null, null);
                }

                if (da.MaxFeesBreached)
                {
                    addDebtorAccountValidation("MF", da.DebtorAccountId, da.MaxFeesExcessValue.ToString(), da.MaxFeesFeeValue.ToString(), da.MaxFeesValue.ToString(), null, null);
                }

                if (da.Reckless)
                {
                    addDebtorAccountValidation("RCK", da.DebtorAccountId, null, null, null, null, null);
                }

                addDebtorAccountValidation("N", da.DebtorAccountId, null, null, null, null, null);
                addDebtorAccountValidation("C", da.DebtorAccountId, null, null, null, null, null);
                addDebtorAccountValidation("A", da.DebtorAccountId, null, null, null, null, null);
            }
        }

        public List<DebtorAccount> SumarizeDebtorAccount()
        {
            List<DebtorAccount> resultValue = new List<DebtorAccount>();

            if (debtorAccountMonthArray.Count > 2)
            {
                foreach (DebtorAccount d in debtorAccountMonthArray[debtorAccountMonthArray.Count - 2])
                {
                    DebtorAccount nd = d;

                    foreach (List<DebtorAccount> dal in debtorAccountMonthArray)
                    {
                        foreach (DebtorAccount da in dal)
                        {
                            if (nd.DebtorAccountId == da.DebtorAccountId)
                            {
                                if ((!nd.InDuplum) && (da.InDuplum))
                                {
                                    nd.InDuplum = da.InDuplum;
                                }
                                if ((!nd.MaxContractInterestBreached) && (da.MaxContractInterestBreached))
                                {
                                    nd.MaxContractInterestBreached = da.MaxContractInterestBreached;
                                }
                                if ((!nd.MaxInterestBreached) && (da.MaxInterestBreached))
                                {
                                    nd.MaxInterestBreached = da.MaxInterestBreached;
                                }
                                if ((!nd.AverageInterestLargerThanContract) && (da.AverageInterestLargerThanContract))
                                {
                                    nd.AverageInterestLargerThanContract = da.AverageInterestLargerThanContract;
                                }
                                if ((!nd.ContractTerm) && (da.ContractTerm))
                                {
                                    nd.ContractTerm = da.ContractTerm;
                                }
                                if ((!nd.BestFitTerm) && (da.BestFitTerm))
                                {
                                    nd.BestFitTerm = da.BestFitTerm;
                                }
                                if ((!nd.Balance) && (da.Balance))
                                {
                                    nd.Balance = da.Balance;
                                }
                                if ((!nd.UnderAdministration) && (da.UnderAdministration))
                                {
                                    nd.UnderAdministration = da.UnderAdministration;
                                }
                                if ((!nd.MaxFeesBreached) && (da.MaxFeesBreached))
                                {
                                    nd.MaxFeesBreached = da.MaxFeesBreached;
                                }
                                if ((!nd.Reckless) && (da.Reckless))
                                {
                                    nd.Reckless = da.Reckless;
                                }
                                if ((!nd.BondMaxLengthExceeded) && (da.BondMaxLengthExceeded))
                                {
                                    if (da.AccountType == Enumerators.AccountType.HomeLoan)
                                    {
                                        nd.BondMaxLengthExceeded = da.BondMaxLengthExceeded;
                                    }
                                    else
                                    {
                                        nd.BondMaxLengthExceeded = false;
                                    }
                                }
                                if ((!nd.CarMaxLengthExceeded) && (da.CarMaxLengthExceeded))
                                {
                                    if (da.AccountType == Enumerators.AccountType.CarLoan)
                                    {
                                        nd.CarMaxLengthExceeded = da.CarMaxLengthExceeded;
                                    }
                                    else
                                    {
                                        nd.CarMaxLengthExceeded = false;
                                    }
                                }
                                if ((!nd.RetailMaxLengthExceeded) && (da.RetailMaxLengthExceeded))
                                {
                                    if (da.AccountType == Enumerators.AccountType.CreditorAccount)
                                    {
                                        nd.RetailMaxLengthExceeded = da.RetailMaxLengthExceeded;
                                    }
                                    else
                                    {
                                        nd.RetailMaxLengthExceeded = false;
                                    }
                                }
                                if ((!nd.GarnishMaxLengthExceeded) && (da.GarnishMaxLengthExceeded))
                                {
                                    if (da.AccountType == Enumerators.AccountType.GarnishOrder)
                                    {
                                        nd.GarnishMaxLengthExceeded = da.GarnishMaxLengthExceeded;
                                    }
                                    else
                                    {
                                        nd.GarnishMaxLengthExceeded = false;
                                    }
                                }
                            }
                        }
                    }

                    resultValue.Add(nd);
                }
            }
            else if (debtorAccountMonthArray.Count > 1)
            {
                resultValue = debtorAccountMonthArray[0];
            }

            return resultValue;
        }

        protected void addDebtorAccountValidation(string validationTypeCode, int debtorAccountId, string value1, string value2, string value3, string value4, string value5)
        {
            System.Data.SqlClient.SqlConnection connection = new SqlConnection(System.Configuration.ConfigurationManager.ConnectionStrings["EliyahuLiteConnectionString"].ConnectionString);

            connection.Open();

            System.Data.SqlClient.SqlCommand command = new SqlCommand("SELECT ValidationTypeId FROM Validation.ValidationType WHERE ValidationTypeCode = @ValidationTypeCode",
                connection);

            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ValidationTypeCode", validationTypeCode));

            int validationTypeId = (int)command.ExecuteScalar();

            command = new SqlCommand("INSERT INTO Debtor.DebtorAccountValidation (DebtorProfileId, DebtorAccountId, AccountValidationTypeId, Value1, Value2, Value3, Value4, Value5, UserGuid) VALUES (@DebtorProfileId, @DebtorAccountId, @AccountValidationTypeId, @Value1, @Value2, @Value3, @Value4, @Value5, @UserGuid)",
                connection);

            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorProfileId", debtorProfileId));
            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorAccountId", debtorAccountId));
            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@AccountValidationTypeId", validationTypeId));
            if (value1 != null)
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value1", value1));
            }
            else
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value1", DBNull.Value));
            }
            if (value2 != null)
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value2", value2));
            }
            else
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value2", DBNull.Value));
            }
            if (value3 != null)
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value3", value3));
            }
            else
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value3", DBNull.Value));
            }
            if (value4 != null)
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value4", value4));
            }
            else
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value4", DBNull.Value));
            }
            if (value5 != null)
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value5", value5));
            }
            else
            {
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Value5", DBNull.Value));
            }
            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@UserGuid", Guid.Empty));

            command.ExecuteNonQuery();

            connection.Close();
        }

        public System.Data.DataTable GetDataTable()
        {
            DataTable debtorAccountTable = new DataTable("DebtorAccount");

            Type debtorAccountType = typeof(DebtorAccount);

            for (int i = 0; i < debtorAccountType.GetProperties().Length; i++)
            {
                DataColumn column = new DataColumn(debtorAccountType.GetProperties()[i].Name);

                Type dataType = debtorAccountType.GetProperties()[i].PropertyType;

                column.DataType = dataType;

                debtorAccountTable.Columns.Add(column);
            }

            foreach (List<DebtorAccount> daArray in debtorAccountMonthArray)
            {
                foreach (DebtorAccount da in daArray)
                {
                    DataRow daRow = debtorAccountTable.NewRow();

                    Type daType = da.GetType();

                    for (int i = 0; i < debtorAccountType.GetProperties().Length; i++)
                    {
                        daRow[debtorAccountType.GetProperties()[i].Name] = daType.GetProperty(debtorAccountType.GetProperties()[i].Name).GetValue(da, null);
                    }

                    debtorAccountTable.Rows.Add(daRow);
                }
            }

            return debtorAccountTable;
        }

        public string GenerateTableCreateScript()
        {
            string resultValue;

            DataTable table = this.GetDataTable();

            resultValue = "CREATE TABLE BestFit.PaymentPlan\r\n(\r\n";
            resultValue += "\tPaymentPlanId\tINT\tPRIMARY KEY\tIDENTITY(1,1)\tNOT NULL,\r\n";
            resultValue += "\tPaymentPlanGuid\tUNIQUEIDENTIFIER\tDEFAULT NEWID(),\r\n";

            for (int i = 0; i < table.Columns.Count; i++)
            {
                DataColumn dc = table.Columns[i];
                resultValue += "\t[" + dc.ColumnName + "]\t";

                if (dc.DataType.FullName == typeof(System.String).FullName)
                {
                    resultValue += "NVARCHAR(MAX)\t";
                }
                else if (dc.DataType.FullName == typeof(System.Boolean).FullName)
                {
                    resultValue += "BIT\t";
                }
                else if (dc.DataType.FullName == typeof(System.Decimal).FullName)
                {
                    resultValue += "DECIMAL(18,4)\t";
                }
                else if (dc.DataType.FullName == typeof(System.DateTime).FullName)
                {
                    resultValue += "DATETIME\t";
                }
                else if ((dc.DataType.FullName == typeof(Enumerators.AccountStatus).FullName) ||
           (dc.DataType == typeof(Enumerators.AccountType)) ||
           (dc.DataType == typeof(Enumerators.AdministrationType)) ||
           (dc.DataType == typeof(Enumerators.InterestType)) ||
           (dc.DataType == typeof(System.Int32)))
                {
                    resultValue += "INT\t";
                }
                else
                {
                    resultValue += "NVARCHAR(MAX)\t";
                }


                if (i != (table.Columns.Count - 1))
                {
                    resultValue += ",";
                }

                resultValue += "\r\n";
            }

            resultValue += " ) ";

            return resultValue;
        }

        public void UpdateTableData()
        {
            System.Data.SqlClient.SqlConnection connection = new System.Data.SqlClient.SqlConnection(System.Configuration.ConfigurationManager.ConnectionStrings["EliyahuLiteConnectionString"].ConnectionString);

            string commandText = "DELETE FROM BestFit.PaymentPlan WHERE DebtorProfileId = @DebtorProfileId AND ((DATEPART(mm, MonthDateTime) = DATEPART(mm, @MonthDateTime)) OR (MonthDateTime >= @MonthDateTime)) ";

            System.Data.SqlClient.SqlCommand command = new System.Data.SqlClient.SqlCommand(commandText, connection);

            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorProfileId", this.debtorProfileId));
            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@MonthDateTime", this.getStartDateTime()));

            connection.Open();

            command.ExecuteNonQuery();

            DataTable table = this.GetDataTable();

            foreach (DataRow dr in table.Rows)
            {
                commandText = "INSERT INTO BestFit.PaymentPlan (";

                for (int i = 0; i < table.Columns.Count; i++)
                {
                    DataColumn dc = table.Columns[i];

                    commandText += "[" + dc.ColumnName + "]";

                    if (i != (table.Columns.Count - 1))
                    {
                        commandText += ", ";
                    }
                }

                commandText += ") VALUES (";

                for (int i = 0; i < table.Columns.Count; i++)
                {
                    DataColumn dc = table.Columns[i];

                    commandText += "@" + dc.ColumnName + "";

                    if (i != (table.Columns.Count - 1))
                    {
                        commandText += ", ";
                    }
                }

                commandText += ")";

                command = new System.Data.SqlClient.SqlCommand(commandText, connection);

                for (int i = 0; i < table.Columns.Count; i++)
                {
                    DataColumn dc = table.Columns[i];

                    command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@" + dc.ColumnName, dr[dc]));
                }

                command.ExecuteNonQuery();
            }

            connection.Close();
        }

        public void UpdateAccountNewSettlement()
        {
            System.Data.SqlClient.SqlConnection connection = new System.Data.SqlClient.SqlConnection(System.Configuration.ConfigurationManager.ConnectionStrings["EliyahuLiteConnectionString"].ConnectionString);

            string commandText = "SELECT DebtorProfileId, DebtorAccountId FROM Debtor.DebtorAccount WHERE DebtorProfileId = @DebtorProfileId";

            System.Data.SqlClient.SqlCommand command = new System.Data.SqlClient.SqlCommand(commandText, connection);

            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorProfileId", this.debtorProfileId));

            connection.Open();

            System.Data.DataTable accountTable = GenerateTableFromReader(command.ExecuteReader(), "debtorAccount");

            foreach (DataRow dr in accountTable.Rows)
            {
                commandText = "SELECT COUNT(PaymentPlanId) FROM BestFit.PaymentPlan WHERE DebtorProfileId = @DebtorProfileId AND DebtorAccountId = @DebtorAccountId";

                command = new System.Data.SqlClient.SqlCommand(commandText, connection);

                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorProfileId", this.debtorProfileId));
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorAccountId", dr["DebtorAccountId"]));

                int bestfitCount = (int)command.ExecuteScalar();

                if (bestfitCount > 0)
                {
                    commandText = "UPDATE Debtor.DebtorAccount SET NewSettlement = ISNULL((SELECT SUM(CurrentInstallmentValue) FROM BestFit.PaymentPlan WHERE DebtorProfileId = @DebtorProfileId AND DebtorAccountId = @DebtorAccountId), 0.00) WHERE DebtorAccountId = @DebtorAccountId";

                    command = new System.Data.SqlClient.SqlCommand(commandText, connection);

                    command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorProfileId", this.debtorProfileId));
                    command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorAccountId", dr["DebtorAccountId"]));

                    command.ExecuteNonQuery();
                }
            }

            connection.Close();
        }

        public void UpdateAccountNewInstallment()
        {
            System.Data.SqlClient.SqlConnection connection = new System.Data.SqlClient.SqlConnection(System.Configuration.ConfigurationManager.ConnectionStrings["EliyahuLiteConnectionString"].ConnectionString);

            string commandText = "SELECT DebtorProfileId, DebtorAccountId FROM Debtor.DebtorAccount WHERE DebtorProfileId = @DebtorProfileId";

            System.Data.SqlClient.SqlCommand command = new System.Data.SqlClient.SqlCommand(commandText, connection);

            command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorProfileId", this.debtorProfileId));

            connection.Open();

            System.Data.DataTable accountTable = GenerateTableFromReader(command.ExecuteReader(), "debtorAccount");

            foreach (DataRow dr in accountTable.Rows)
            {
                commandText = "SELECT COUNT(PaymentPlanId) FROM BestFit.PaymentPlan WHERE DebtorProfileId = @DebtorProfileId AND DebtorAccountId = @DebtorAccountId";

                command = new System.Data.SqlClient.SqlCommand(commandText, connection);

                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorProfileId", this.debtorProfileId));
                command.Parameters.Add(new System.Data.SqlClient.SqlParameter("@DebtorAccountId", dr["DebtorAccountId"]));

                int bestfitCount = (int)command.ExecuteScalar();

                if (bestfitCount > 0)
                {
                    commandText = string.Format("UPDATE Debtor.DebtorAccount SET NewInstallment = ISNULL((SELECT TOP 1 CurrentInstallmentValue FROM BestFit.PaymentPlan WHERE DebtorProfileID = {0} AND DebtorAccountId = {1} AND [MonthDateTime] > GETDATE() ORDER BY [Month] ASC), 0.00) WHERE DebtorAccountId = {1}", dr["DebtorProfileId"], dr["DebtorAccountId"]);

                    command = new System.Data.SqlClient.SqlCommand(commandText, connection);

                    command.ExecuteNonQuery();
                }
            }

            connection.Close();
        }

        public DataTable GenerateTableFromReader(SqlDataReader Reader, string TableName)
        {
            DataTable resultValue = new DataTable(TableName);

            for (int i = 0; i < Reader.FieldCount; i++)
            {
                DataColumn column = new DataColumn(Reader.GetName(i), Reader.GetFieldType(i));

                resultValue.Columns.Add(column);
            }

            while (Reader.Read())
            {
                DataRow row = resultValue.NewRow();

                for (int i = 0; i < Reader.FieldCount; i++)
                {
                    DataColumn column = resultValue.Columns[Reader.GetName(i)];

                    row[column] = Reader.GetValue(i);
                }

                resultValue.Rows.Add(row);
            }

            Reader.Close();

            return resultValue;
        }

        public void Reset()
        {
            List<DebtorAccount> debtorAccountListClone = debtorAccountArray;
            List<List<DebtorAccount>> debtorAccountMonthClone = new List<List<DebtorAccount>>();

            debtorAccountMonthClone.Add(debtorAccountListClone);

            debtorAccountMonthArray = debtorAccountMonthClone;

            currentMonth = 0;
        }

        public List<DebtorAccount> DebtorAccountArray
        {
            get { return debtorAccountArray; }
            set { debtorAccountArray = value; }
        }

        public List<List<DebtorAccount>> DebtorAccountMonthArray
        {
            get { return debtorAccountMonthArray; }
            set { debtorAccountMonthArray = value; }
        }

        public decimal AvailableFunds
        {
            get { return availableFunds; }
            set { availableFunds = value; }
        }

        public int CurrentMonth
        {
            get { return currentMonth; }
            set { currentMonth = value; }
        }

        protected bool hasOutstandingValue()
        {
            return (calculateTotalSettlement() > 0);
        }

        public Dictionary<string, string> GeneralSettings
        {
            get { return generalSettings; }
            set { generalSettings = value; }
        }

        public void GetGeneralSettings()
        {
            System.Data.SqlClient.SqlConnection connection = new System.Data.SqlClient.SqlConnection(System.Configuration.ConfigurationManager.ConnectionStrings["EliyahuLiteConnectionString"].ConnectionString);

            System.Data.SqlClient.SqlCommand command = new System.Data.SqlClient.SqlCommand("SELECT GeneralSettingsCode, GeneralSettingsValue FROM System.GeneralSettings ", connection);

            connection.Open();

            System.Data.SqlClient.SqlDataReader reader = command.ExecuteReader();

            while (reader.Read())
            {
                generalSettings.Add((string)reader["GeneralSettingsCode"], (string)reader["GeneralSettingsValue"]);
            }

            reader.Close();
            connection.Close();
        }

        protected DateTime getStartDateTime()
        {
            DateTime resultValue = DateTime.Today;

            if ((debtorAccountMonthArray.Count > 0) && (debtorAccountMonthArray[0].Count > 0))
            {
                resultValue = debtorAccountMonthArray[0][0].MonthDateTime;
            }

            return resultValue;
        }

        protected decimal calculateTotalSettlement()
        {
            decimal outstandingValue = 0.00M;

            for (int i = 0; i < debtorAccountMonthArray[debtorAccountMonthArray.Count - 1].Count; i++)
            {
                outstandingValue += debtorAccountMonthArray[debtorAccountMonthArray.Count - 1][i].CurrentSettlementValue;
            }

            return outstandingValue;
        }

        protected decimal calculateTotalInstallment()
        {
            decimal outstandingValue = 0.00M;

            for (int i = 0; i < debtorAccountMonthArray[debtorAccountMonthArray.Count - 1].Count; i++)
            {
                outstandingValue += (debtorAccountMonthArray[debtorAccountMonthArray.Count - 1][i].CurrentInstallmentValue == 0.00M) ? debtorAccountMonthArray[debtorAccountMonthArray.Count - 1][i].OriginalInstallmentValue : debtorAccountMonthArray[debtorAccountMonthArray.Count - 1][i].CurrentInstallmentValue;
            }

            return outstandingValue;
        }

        protected decimal calculateTotalOriginalInstallment()
        {
            decimal outstandingValue = 0.00M;

            for (int i = 0; i < debtorAccountMonthArray[debtorAccountMonthArray.Count - 1].Count; i++)
            {
                outstandingValue += debtorAccountMonthArray[debtorAccountMonthArray.Count - 1][i].OriginalInstallmentValue;
            }

            return outstandingValue;
        }

        protected decimal calculateTotalInterest()
        {
            decimal outstandingValue = 0.00M;

            for (int i = 0; i < debtorAccountMonthArray[debtorAccountMonthArray.Count - 1].Count; i++)
            {
                outstandingValue += debtorAccountMonthArray[debtorAccountMonthArray.Count - 1][i].CurrentInterestRate;
            }

            return outstandingValue;
        }

        public int DebtorProfileId
        {
            get { return debtorProfileId; }
            set { debtorProfileId = value; }
        }

        public decimal MonthlyFees
        {
            get { return monthlyFees; }
            set { monthlyFees = value; }
        }

        public Enumerators.AdministrationType AdministrationType
        {
            get { return administrationType; }
            set { administrationType = value; }
        }
    }
}
